
/*
A simple C++ wrapper around WIN32 tray icon functions
Copyright (C) 2000 Antti Markus (antti@pld.ttu.ee)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "trayicon.h"

CTrayIcon::CTrayIcon(void)
{
  Data.cbSize=0;
  Data.hWnd=0;
  Data.uID=0;
  Data.uFlags=0;
  Data.uCallbackMessage=0;
  Data.hIcon=0;
  Data.szTip[0]='\0';
  uTrayed=0;
}

CTrayIcon::CTrayIcon(const HWND hwnd, const UINT uMsg, const UINT uID, 
                      const HICON hIcon, const char* pTip)
{
  Data.cbSize=sizeof(NOTIFYICONDATA);
  Data.hWnd=hwnd;   //notification messages are sent to window hwnd
  Data.uID=uID;     //identifier of the taskbar icon
  Data.uCallbackMessage=uMsg; //notification message sent when mouse event occurs
  Data.hIcon=hIcon; //icon handle
  if(pTip)          //if tooltip is specified
  {
    strcpy(Data.szTip, pTip);
    Data.uFlags=NIF_ICON | NIF_MESSAGE | NIF_TIP;
  }
  else
    Data.uFlags=NIF_ICON | NIF_MESSAGE;
  uTrayed=0;
}

CTrayIcon::~CTrayIcon(void)
{
  if(uTrayed)
  {
    Shell_NotifyIcon(NIM_DELETE, &Data);
    uTrayed=0;
  }
}

void CTrayIcon::Init(const HWND hwnd, const UINT uMsg, const UINT uID, 
                     const HICON hIcon, const char* pTip)
{
  Data.cbSize=sizeof(NOTIFYICONDATA);
  Data.hWnd=hwnd;
  Data.uID=uID;
  Data.uCallbackMessage=uMsg;
  Data.hIcon=hIcon;
  if(pTip)
  {
    strcpy(Data.szTip, pTip);
    Data.uFlags=NIF_ICON | NIF_MESSAGE | NIF_TIP;
  }
  else
    Data.uFlags=NIF_ICON | NIF_MESSAGE;
}

void CTrayIcon::Put(void)
{
  if(!uTrayed)
  {
    Shell_NotifyIcon(NIM_ADD, &Data);
    uTrayed=1;
  }
}

void CTrayIcon::Remove(void)
{
  if(uTrayed)
  {
    Shell_NotifyIcon(NIM_DELETE, &Data);
    uTrayed=0;
  }
}

void CTrayIcon::Modify(const HICON hIcon)
{
  if(uTrayed)
  {
    Data.hIcon=hIcon;
    Shell_NotifyIcon(NIM_MODIFY, &Data);
  }
}

void CTrayIcon::Modify(const char* pTip)
{
  if(uTrayed)
  {
    if(pTip)
    {
      if(!Data.szTip[0])  //if previously no tooltip
        Data.uFlags=Data.uFlags | NIF_TIP;
      strcpy(Data.szTip, pTip);
    }
    else
      Data.uFlags=NIF_ICON | NIF_MESSAGE;
    Shell_NotifyIcon(NIM_MODIFY, &Data);
  }
}