#include <windows.h>
#include <stdio.h>

#define WP_RESTOREPROC 0x8001

__declspec( dllexport ) int InitModule(HINSTANCE hInstance, HWND parent, char* config);
__declspec( dllexport ) void QuitModule();
__declspec( dllexport ) DLGPROC DialogConfigProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
__declspec( dllexport ) char* QueryConfigId();
__declspec( dllexport ) char* QueryName();
__declspec( dllexport ) void ShowModuleWindow(DWORD show);

LRESULT CALLBACK WndProc(HWND hwnd, int msg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam);

HOOKPROC HookFunction(int code, WPARAM wParam, LPARAM lParam);

static HBITMAP MiddleBmp;
static char MiddleBmpPath[256];
static HBITMAP LeftBmp;
static char LeftBmpPath[256];
static HBITMAP RightBmp;
static char RightBmpPath[256];
static HBITMAP LeftEdgeBmp;
static char LeftEdgeBmpPath[256];
static HBITMAP RightEdgeBmp;
static char RightEdgeBmpPath[256];
static HBITMAP BottomEdgeBmp;
static char BottomEdgeBmpPath[256];

WNDPROC OldWndProc=NULL;

#pragma data_seg ("STATIC")
HBITMAP MiddleBmp=NULL;
char MiddleBmpPath[256] = "";
HBITMAP LeftBmp=NULL;
char LeftBmpPath[256] = "";
HBITMAP RightBmp=NULL;
char RightBmpPath[256] = "";
HBITMAP LeftEdgeBmp=NULL;
char LeftEdgeBmpPath[256] = "";
HBITMAP RightEdgeBmp=NULL;
char RightEdgeBmpPath[256] = "";
HBITMAP BottomEdgeBmp=NULL;
char BottomEdgeBmpPath[256] = "";
#pragma data_seg ()

HINSTANCE hinst=NULL;
HWND parent=NULL;
char* szAppName = "Garbonzo";
char cfgfile[MAX_PATH] = "";
HHOOK hook=NULL;

int InitModule(HINSTANCE hInstance, HWND Parent, char* config)
{
	// Save all incoming settings
	hinst = hInstance;
	parent = Parent;
	strcpy(cfgfile, config);
	WriteProfileString(szAppName, "cfgfile", cfgfile);

	hook = SetWindowsHookEx(WH_SHELL, (HOOKPROC)HookFunction, hInstance, 0);

	return 1;
}

char* QueryConfigId() { return NULL; } //return MAKEINTRESOURCE(IDD_CONFIG); }
char* QueryName() { return szAppName; }
void ShowModuleWindow(DWORD show) { return; }

HOOKPROC HookFunction(int code, WPARAM wParam, LPARAM lParam)
{
	if (code < 0) 
	{
		return (HOOKPROC)CallNextHookEx(hook, code, wParam, lParam);
	}

	switch (code)
	{
		case HSHELL_WINDOWCREATED:
		{
			DWORD style = GetWindowLong((HWND)wParam, GWL_STYLE);
			int y = style & WS_CAPTION;
			int z = style & WS_CHILD;
			if (y && !z)
			{
				OldWndProc = (WNDPROC)SetWindowLong((HWND)wParam, GWL_WNDPROC, (LONG)WndProc);
			}
		}
		return 0;
	}

	return 0;
}

LRESULT CALLBACK WndProc(HWND hwnd, int msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WP_RESTOREPROC:
		{
			SetWindowLong(hwnd, GWL_WNDPROC, (LONG)OldWndProc);
			DeleteObject(MiddleBmp);
			DeleteObject(LeftBmp);
			DeleteObject(RightBmp);
			MiddleBmp = LeftBmp = RightBmp = NULL;
			PostMessage(hwnd, WM_NCPAINT, 1, 0);
		}
		break;

		case WM_NCHITTEST:
		{
			int x = LOWORD(lParam);
			int y = HIWORD(lParam);
			RECT wr;
			GetWindowRect(hwnd, &wr);

			x -= wr.left;
			y -= wr.top;
			if (y <= GetSystemMetrics(SM_CYFRAME)) return HTTOP;
			else if (y <= (GetSystemMetrics(SM_CYFRAME) + GetSystemMetrics(SM_CYSIZE))) 
			{
				if (x >= 6 && x <= 19 && y >= 6 && y <= 19)
				{
					return HTCLOSE;
				}
				else
				{
					return HTCAPTION;
				}
			}
			else if (x <= GetSystemMetrics(SM_CXFRAME)) return HTLEFT;
			else if (x >= (wr.right - wr.left - GetSystemMetrics(SM_CXFRAME))) return HTRIGHT;
			else if (y >= (wr.bottom - wr.top - GetSystemMetrics(SM_CYFRAME))) return HTBOTTOM;
		}
		break;

		case WM_WINDOWPOSCHANGED:
		case WM_ACTIVATE:
		case WM_SETTEXT:
		{
			PostMessage(hwnd, WM_NCPAINT, 0, 0);  break;
		}
		break;

		case WM_NCLBUTTONDOWN:
		{
			if (wParam == HTCLOSE) SendMessage(hwnd, WM_SYSCOMMAND, SC_CLOSE, 0);
		}
		break;

		case WM_NCPAINT:
		{
			HDC hdc = GetWindowDC(hwnd);
			HDC buf = CreateCompatibleDC(hdc);
			HDC tdc = CreateCompatibleDC(hdc);
			HBITMAP bufbmp, oldbufbmp;
			HBITMAP oldtdcbmp;
			BITMAP tbmp;
			RECT rc1, rc2;
			HRGN rgn = CreateRectRgn(0, 0, 0, 0);
			int xframe, yframe, ysize, width, height, ymenu;
			char window_text[256] = "";
			int START_TILE=-1, STOP_TILE=-1;
			int x=0;
		
			GetWindowText(hwnd, window_text, 256);
			GetWindowRect(hwnd, &rc2);
			width = rc2.right - rc2.left;
			height = rc2.bottom - rc2.top;
			xframe = GetSystemMetrics(SM_CXFRAME);
			yframe = GetSystemMetrics(SM_CYFRAME);
			ysize = GetSystemMetrics(SM_CYSIZE);
			ymenu = GetSystemMetrics(SM_CYMENU);
			
			SetRectRgn(rgn, yframe, xframe+ysize, width-yframe, xframe+ysize+20);
			DefWindowProc(hwnd, msg, (WPARAM)rgn, 1);

			//GetProfileString(szAppName, "cfgfile", "winpanel.cfg", cfgfile, 256);
			strcpy(cfgfile, "c:\\winpanel\\winpanel.cfg");
			START_TILE = GetPrivateProfileInt(szAppName, "StartTile", 19, cfgfile);
			STOP_TILE = GetPrivateProfileInt(szAppName, "StopTile", 19, cfgfile);
			STOP_TILE = width - STOP_TILE;
			
			if (!MiddleBmp) 
			{
				GetPrivateProfileString(szAppName, "MiddleBmp", "c:\\winpanel\\images\\g-middle.bmp", MiddleBmpPath, 256, cfgfile);
				MiddleBmp = LoadImage(NULL, MiddleBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			}

			if (!LeftBmp) 
			{
				GetPrivateProfileString(szAppName, "LeftBmp", "c:\\winpanel\\images\\g-left.bmp", LeftBmpPath, 256, cfgfile);
				LeftBmp = LoadImage(NULL, LeftBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			}

			if (!RightBmp) 
			{
				GetPrivateProfileString(szAppName, "RightBmp", "c:\\winpanel\\images\\g-right.bmp", RightBmpPath, 256, cfgfile);
				RightBmp = LoadImage(NULL, RightBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			}

			if (!LeftEdgeBmp) 
			{
				GetPrivateProfileString(szAppName, "LeftEdgeBmp", "c:\\winpanel\\images\\g-leftside.bmp", LeftEdgeBmpPath, 256, cfgfile);
				LeftEdgeBmp = LoadImage(NULL, LeftEdgeBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			}

			if (!RightEdgeBmp) 
			{
				GetPrivateProfileString(szAppName, "RightEdgeBmp", "c:\\winpanel\\images\\g-rightside.bmp", RightEdgeBmpPath, 256, cfgfile);
				RightEdgeBmp = LoadImage(NULL, RightEdgeBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			}

			if (!BottomEdgeBmp) 
			{
				GetPrivateProfileString(szAppName, "BottomEdgeBmp", "c:\\winpanel\\images\\g-bottom.bmp", BottomEdgeBmpPath, 256, cfgfile);
				BottomEdgeBmp = LoadImage(NULL, BottomEdgeBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
			}

			bufbmp = CreateCompatibleBitmap(hdc, width, height);
			oldbufbmp = SelectObject(buf, bufbmp);

			oldtdcbmp = SelectObject(tdc, LeftBmp);
			GetObject(LeftBmp, sizeof(tbmp), &tbmp);
			BitBlt(buf, 0, 0, tbmp.bmWidth, tbmp.bmHeight, tdc, 0, 0, SRCCOPY);
			SelectObject(tdc, oldtdcbmp);

			oldtdcbmp = SelectObject(tdc, MiddleBmp);
			GetObject(MiddleBmp, sizeof(tbmp), &tbmp);
			for (x=START_TILE; x < STOP_TILE; x += tbmp.bmWidth)
			{
				BitBlt(buf, x, 0, tbmp.bmWidth, tbmp.bmHeight, tdc, 0, 0, SRCCOPY);
			}
			SelectObject(tdc, oldtdcbmp);

			oldtdcbmp = SelectObject(tdc, RightBmp);
			GetObject(RightBmp, sizeof(tbmp), &tbmp);
			BitBlt(buf, STOP_TILE, 0, tbmp.bmWidth, tbmp.bmHeight, tdc, 0, 0, SRCCOPY);
			SelectObject(tdc, oldtdcbmp);

			SetBkMode(buf, TRANSPARENT);
			SetRect(&rc1, 0, ((ysize + xframe) / 2) - 8, width, ysize + xframe);
			DrawText( buf, (LPSTR)window_text, -1, (LPRECT)&rc1, DT_CENTER);

			BitBlt(hdc, 0, 0, width, xframe + ysize, buf, 0, 0, SRCCOPY);

			oldtdcbmp = SelectObject(tdc, LeftEdgeBmp);
			GetObject(LeftEdgeBmp, sizeof(tbmp), &tbmp);
			for (x=xframe+ysize; x < height; x += tbmp.bmHeight)
			{
				BitBlt(buf, 0, x, tbmp.bmWidth, tbmp.bmHeight, tdc, 0, 0, SRCCOPY);
			}
			BitBlt(hdc, 0, xframe+ysize, tbmp.bmWidth, height - xframe - ysize, buf, 0, xframe+ysize, SRCCOPY);
			SelectObject(tdc, oldtdcbmp);

			oldtdcbmp = SelectObject(tdc, RightEdgeBmp);
			GetObject(RightEdgeBmp, sizeof(tbmp), &tbmp);
			for (x=xframe+ysize; x < height; x += tbmp.bmHeight)
			{
				BitBlt(buf, width - yframe, x, tbmp.bmWidth, tbmp.bmHeight, tdc, 0, 0, SRCCOPY);
			}
			BitBlt(hdc, width-yframe, xframe+ysize, tbmp.bmWidth, height - xframe - ysize, buf, 0, xframe+ysize, SRCCOPY);
			SelectObject(tdc, oldtdcbmp);

			oldtdcbmp = SelectObject(tdc, BottomEdgeBmp);
			GetObject(BottomEdgeBmp, sizeof(tbmp), &tbmp);
			for (x=0; x < width; x += tbmp.bmWidth)
			{
				BitBlt(buf, x, height-xframe, tbmp.bmWidth, tbmp.bmHeight, tdc, 0, 0, SRCCOPY);
			}
			BitBlt(hdc, 0, height-xframe, width, tbmp.bmHeight, buf, 0, height-xframe, SRCCOPY);
			SelectObject(tdc, oldtdcbmp);
		
			SelectObject(buf, oldbufbmp);
			DeleteObject(bufbmp);
			DeleteDC(buf);
			DeleteObject(oldbufbmp);
			DeleteDC(tdc);
			DeleteObject(oldtdcbmp);
			ReleaseDC(hwnd, hdc);
		}			
		return 0;
	}

	return CallWindowProc(OldWndProc, hwnd, msg, wParam, lParam);
}

void QuitModule()
{
	EnumWindows(EnumWindowsProc, 0);	
	UnhookWindowsHookEx(hook);
}

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	PostMessage(hwnd, WP_RESTOREPROC, 0, 0);
	return TRUE;
}