#include <windows.h>
#include <stdio.h>
#include "resource.h"

__declspec( dllexport ) int InitModule(HINSTANCE hInstance, HWND parent, char* config);
__declspec( dllexport ) void QuitModule();
__declspec( dllexport ) DLGPROC DialogConfigProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
__declspec( dllexport ) char* QueryConfigId();
__declspec( dllexport ) char* QueryName();
__declspec( dllexport ) void ShowModuleWindow(DWORD show);

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK TaskWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

void AddWindow(HWND hwnd);
BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam);
void LoadSetup();
void ResizeAllWindows();
void WPChooseFont(HWND hwnd, char* CurrentFace, int CurrentHeight, COLORREF CurrentColor, char* DestFace, char* DestColor, char* DestHeight);

HINSTANCE hinst=NULL;
HWND parent=NULL;
char cfgfile[MAX_PATH] = "";
char* szAppName = "WPTasks";
char IMAGE_DIR[MAX_PATH] = "";

HWND hwndMain=NULL;
int X, Y, W, H, COUNT=0, TASK_HEIGHT=20, TASK_WIDTH=60, VSPACING=0, HSPACING=0, MAXROW=-1;
HBITMAP ActiveBmp=NULL, InactiveBmp=NULL, MinBmp=NULL, BgBmp=NULL;
char ActiveBmpPath[256] = "", InactiveBmpPath[256] = "", MinBmpPath[256] = "", BgBmpPath[256] = "";

struct Window
{
	HWND hwnd, bhwnd;
	char windowtext[256];
	int index;
};

struct Font
{
	HFONT font;
	char face[256];
	int height;
	COLORREF color;
} FontActive, FontInactive, FontMin;

int InitModule(HINSTANCE hInstance, HWND Parent, char* config)
{
	// Save all incoming settings
	hinst = hInstance;
	parent = Parent;
	strcpy(cfgfile, config);

	if (parent)
	{
		WNDCLASS wc;
		
		memset(&wc,0,sizeof(wc));
		wc.hInstance = hInstance;
		wc.lpfnWndProc = WndProc;
		wc.lpszClassName = szAppName;
		wc.hCursor = LoadCursor(NULL, IDC_ARROW);
		RegisterClass(&wc);

		wc.lpfnWndProc = TaskWndProc;
		wc.lpszClassName = "WPTaskWindow";
		RegisterClass(&wc);

		LoadSetup();
	}

	return 1;
}

void QuitModule()
{
	// Unregister windows classes etc...
	KillTimer(hwndMain, 0);
	UnregisterClass(szAppName, hinst);
	UnregisterClass("WPTaskWindow", hinst);
	DestroyWindow(hwndMain);
	DeleteObject(BgBmp);
	DeleteObject(ActiveBmp);
	DeleteObject(InactiveBmp);
	DeleteObject(MinBmp);
	DeleteObject(FontActive.font);
	DeleteObject(FontInactive.font);
	DeleteObject(FontMin.font);
}

char* QueryConfigId() { return MAKEINTRESOURCE(IDD_CONFIG); }
char* QueryName() { return szAppName; }
void ShowModuleWindow(DWORD show) 
{ 
	ShowWindow(hwndMain, show);
	if (show == SW_SHOW)
	{
		HWND hwnd=NULL;
		while ((hwnd = FindWindowEx(hwndMain, hwnd, "WPTaskWindow", NULL)) != NULL)
		{
			InvalidateRect(hwnd, NULL, TRUE);
		}
	}
}

void LoadSetup()
{
	char temp[256] = "";
	char *endstr;

	X = GetPrivateProfileInt(szAppName, "X", 0, cfgfile);
	Y = GetPrivateProfileInt(szAppName, "Y", 0, cfgfile);
	W = GetPrivateProfileInt(szAppName, "W", 0, cfgfile);
	H = GetPrivateProfileInt(szAppName, "H", 0, cfgfile);
	MAXROW = GetPrivateProfileInt(szAppName, "MaxPerRow", -1, cfgfile);
	VSPACING = GetPrivateProfileInt(szAppName, "VSpacing", 0, cfgfile);
	HSPACING = GetPrivateProfileInt(szAppName, "HSpacing", 0, cfgfile);

	TASK_HEIGHT = GetPrivateProfileInt(szAppName, "TaskHeight", 20, cfgfile);
	TASK_WIDTH = GetPrivateProfileInt(szAppName, "TaskWidth", 60, cfgfile);
	GetPrivateProfileString("Main", "ImageDirectory", "c:\\winpanel\\images\\", IMAGE_DIR, MAX_PATH, cfgfile);
	if (IMAGE_DIR[strlen(IMAGE_DIR)] != '\\')
	{
		IMAGE_DIR[strlen(IMAGE_DIR)] = '\\';
		IMAGE_DIR[strlen(IMAGE_DIR)+1] = '\0';
	}
	
	GetPrivateProfileString(szAppName, "BgBmp", "", temp, 256, cfgfile);
	sprintf(BgBmpPath, "%s%s", IMAGE_DIR, temp);
	BgBmp = LoadImage(NULL, BgBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

	GetPrivateProfileString(szAppName, "ActiveBmp", "", temp, 256, cfgfile);
	sprintf(ActiveBmpPath, "%s%s", IMAGE_DIR, temp);
	ActiveBmp = LoadImage(NULL, ActiveBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

	GetPrivateProfileString(szAppName, "InactiveBmp", "", temp, 256, cfgfile);
	sprintf(InactiveBmpPath, "%s%s", IMAGE_DIR, temp);
	InactiveBmp = LoadImage(NULL, InactiveBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	
	GetPrivateProfileString(szAppName, "MinBmp", "", temp, 256, cfgfile);
	sprintf(MinBmpPath, "%s%s", IMAGE_DIR, temp);
	MinBmp = LoadImage(NULL, MinBmpPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

	GetPrivateProfileString(szAppName, "ActiveFontColor", "0", temp, 256, cfgfile);
	FontActive.color = strtol( temp, &endstr, 16 );
	FontActive.height = GetPrivateProfileInt(szAppName, "ActiveFontHeight", 12, cfgfile);
	GetPrivateProfileString(szAppName, "ActiveFontFace", "Arial", FontActive.face, 256, cfgfile);
	FontActive.font = CreateFont(FontActive.height, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, FontActive.face);

	GetPrivateProfileString(szAppName, "InactiveFontColor", "0", temp, 256, cfgfile);
	FontInactive.color = strtol( temp, &endstr, 16 );
	FontInactive.height = GetPrivateProfileInt(szAppName, "InactiveFontHeight", 12, cfgfile);
	GetPrivateProfileString(szAppName, "InactiveFontFace", "Arial", FontInactive.face, 256, cfgfile);
	FontInactive.font = CreateFont(FontInactive.height, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, FontInactive.face);

	GetPrivateProfileString(szAppName, "MinFontColor", "0", temp, 256, cfgfile);
	FontMin.color = strtol( temp, &endstr, 16 );
	FontMin.height = GetPrivateProfileInt(szAppName, "MinFontHeight", 12, cfgfile);
	GetPrivateProfileString(szAppName, "MinFontFace", "Arial", FontMin.face, 256, cfgfile);
	FontMin.font = CreateFont(FontMin.height, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, FontMin.face);

	hwndMain = CreateWindow(szAppName, szAppName, WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN, X, Y, W, H, parent, NULL, hinst, 0);

	EnumWindows(EnumWindowsProc, 0);
	SetTimer(hwndMain, 0, 500, NULL);
	SetTimer(hwndMain, 1, 1000, NULL);
}

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	DWORD ws = GetWindowLong(hwnd, GWL_STYLE);
	DWORD wsex = GetWindowLong(hwnd, GWL_EXSTYLE);

	if (ws & WS_VISIBLE && !(wsex & WS_EX_TOOLWINDOW) && ws & WS_CAPTION && ws & WS_SYSMENU)
	{
		HWND find = NULL;
		while ((find = FindWindowEx(hwndMain, find, "WPTaskWindow", NULL)) != NULL)
		{
			struct Window* w = (struct Window*)GetWindowLong(find, GWL_USERDATA);
			if (hwnd == w->bhwnd) 
			{ 
				GetWindowText(w->bhwnd, w->windowtext, 256);
				InvalidateRect(w->hwnd, NULL, TRUE); 
				return TRUE; 
			}
		}
		AddWindow(hwnd);
	}

	return TRUE;
}

DLGPROC DialogConfigProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_INITDIALOG:
		{
			SetDlgItemInt(hwnd, IDC_X, X, TRUE);
			SetDlgItemInt(hwnd, IDC_Y, Y, TRUE);
			SetDlgItemInt(hwnd, IDC_WIDTH, W, FALSE);
			SetDlgItemInt(hwnd, IDC_HEIGHT, H, FALSE);
			SetDlgItemInt(hwnd, IDC_TASK_WIDTH, TASK_WIDTH, FALSE);
			SetDlgItemInt(hwnd, IDC_TASK_HEIGHT, TASK_HEIGHT, FALSE);
			SetDlgItemText(hwnd, IDC_BMP_BG, BgBmpPath);
			SetDlgItemText(hwnd, IDC_BMP_ACTIVE, ActiveBmpPath);
			SetDlgItemText(hwnd, IDC_BMP_INACTIVE, InactiveBmpPath);
			SetDlgItemText(hwnd, IDC_BMP_MIN, MinBmpPath);
		}
		break;

		case WM_COMMAND:
		{
			switch (HIWORD(wParam))
			{
				case BN_CLICKED:
				{
					switch (LOWORD(wParam))
					{
						case IDC_FONT_ACTIVE: WPChooseFont(hwnd, FontActive.face, FontActive.height, FontActive.color, "ActiveFontFace", "ActiveFontColor", "ActiveFontHeight"); break;
						case IDC_FONT_INACTIVE: WPChooseFont(hwnd, FontInactive.face, FontInactive.height, FontInactive.color, "InactiveFontFace", "InactiveFontColor", "InactiveFontHeight"); break;
						case IDC_FONT_MIN: WPChooseFont(hwnd, FontMin.face, FontMin.height, FontMin.color, "MinFontFace", "MinFontColor", "MinFontHeight"); break;
					}
				}
				break;
			}
		}
		break;

		case WM_DESTROY:
		{
			char temp[256] = "";
			GetDlgItemText(hwnd, IDC_X, temp, 256);
			WritePrivateProfileString(szAppName, "X", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_Y, temp, 256);
			WritePrivateProfileString(szAppName, "Y", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_WIDTH, temp, 256);
			WritePrivateProfileString(szAppName, "W", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_HEIGHT, temp, 256);
			WritePrivateProfileString(szAppName, "H", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_TASK_WIDTH, temp, 256);
			WritePrivateProfileString(szAppName, "TaskWidth", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_TASK_HEIGHT, temp, 256);
			WritePrivateProfileString(szAppName, "TaskHeight", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_BMP_BG, temp, 256);
			WritePrivateProfileString(szAppName, "BgBmp", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_BMP_ACTIVE, temp, 256);
			WritePrivateProfileString(szAppName, "ActiveBmp", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_BMP_INACTIVE, temp, 256);
			WritePrivateProfileString(szAppName, "InactiveBmp", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_BMP_MIN, temp, 256);
			WritePrivateProfileString(szAppName, "MinBmp", temp, cfgfile);
		}
		break;
	}

	return FALSE;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd, &ps);
			HDC buf = CreateCompatibleDC(NULL);
			HBITMAP oldbufbmp;
			oldbufbmp = (HBITMAP)SelectObject(buf, BgBmp);
			BitBlt(hdc, 0, 0, W, H, buf, 0, 0, SRCCOPY);
			EndPaint(hwnd, &ps);
			SelectObject(buf, oldbufbmp);
			DeleteDC(buf);
			DeleteObject(oldbufbmp);
		}
		break;

		case WM_TIMER:
		{
			switch (wParam)
			{
				case 0:
				{
					EnumWindows(EnumWindowsProc, 0);
				}
				break;

				case 1:
				{
					HWND find = NULL;
					while ((find = FindWindowEx(hwndMain, find, "WPTaskWindow", NULL)) != NULL)
					{
						struct Window* w = (struct Window*)GetWindowLong(find, GWL_USERDATA);
						if (!IsWindow(w->bhwnd) || !IsWindowVisible(w->bhwnd)) 
						{ 
							DestroyWindow(w->hwnd);
							COUNT--; 
							ResizeAllWindows(); 
						}
					}
				}
				break;
			}
		}
		break;
	}

	return DefWindowProc(hwnd, msg, wParam, lParam);
}

LRESULT CALLBACK TaskWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd, &ps);
			HDC buf = CreateCompatibleDC(NULL);
			HDC back = CreateCompatibleDC(NULL);
			HBITMAP bufbmp, oldbmp, oldbackbmp;
			RECT r;
			struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);
			BITMAP bmp;

			GetClientRect(hwnd, &r);

			SetBkMode(buf, TRANSPARENT);
			if (w->bhwnd == GetForegroundWindow()) 
			{
				oldbackbmp = SelectObject(back, ActiveBmp);
				GetObject(ActiveBmp, sizeof(bmp), &bmp);
				bufbmp = CreateCompatibleBitmap(hdc, bmp.bmWidth, bmp.bmHeight);
				oldbmp = SelectObject(buf, bufbmp);
				StretchBlt(buf, 0, 0, r.right, r.bottom, back, 0, 0, bmp.bmWidth, bmp.bmHeight, SRCCOPY);
				SetTextColor(buf, FontActive.color);
				SelectObject(buf, FontActive.font);
				TextOut(buf, 5, r.bottom / 2 - (FontActive.height/2), w->windowtext, strlen(w->windowtext));
			}
			else if (IsIconic(w->bhwnd)) 
			{
				oldbackbmp = SelectObject(back, MinBmp);
				GetObject(InactiveBmp, sizeof(bmp), &bmp);
				bufbmp = CreateCompatibleBitmap(hdc, bmp.bmWidth, bmp.bmHeight);
				oldbmp = SelectObject(buf, bufbmp);
				StretchBlt(buf, 0, 0, r.right, r.bottom, back, 0, 0, bmp.bmWidth, bmp.bmHeight, SRCCOPY);
				SetTextColor(buf, FontInactive.color);
				SelectObject(buf, FontInactive.font);
				TextOut(buf, 5, r.bottom / 2 - (FontInactive.height/2), w->windowtext, strlen(w->windowtext));
			}
			else
			{
				oldbackbmp = SelectObject(back, InactiveBmp);
				GetObject(MinBmp, sizeof(bmp), &bmp);
				bufbmp = CreateCompatibleBitmap(hdc, bmp.bmWidth, bmp.bmHeight);
				oldbmp = SelectObject(buf, bufbmp);
				StretchBlt(buf, 0, 0, r.right, r.bottom, back, 0, 0, bmp.bmWidth, bmp.bmHeight, SRCCOPY);
				SetTextColor(buf, FontMin.color);
				SelectObject(buf, FontMin.font);
				TextOut(buf, 5, r.bottom / 2 - (FontMin.height/2), w->windowtext, strlen(w->windowtext));
			}
			
			BitBlt(hdc, 0, 0, r.right, r.bottom, buf, 0, 0, SRCCOPY);

			SelectObject(back, oldbackbmp);
			SelectObject(buf, oldbmp);
			DeleteObject(bufbmp);
			DeleteDC(buf);
			DeleteDC(back);
			DeleteObject(oldbmp);
			DeleteObject(oldbackbmp);

			EndPaint(hwnd, &ps);
			w=NULL;
			free(w);
		}
		break;

		case WM_LBUTTONUP:
		{
			struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);
			
			if (!IsWindow(w->bhwnd) || !IsWindowVisible(w->bhwnd))
			{
				DestroyWindow(w->hwnd);
				COUNT--; 
				ResizeAllWindows(); 
			}

			if (IsIconic(w->bhwnd)) ShowWindow(w->bhwnd, SW_RESTORE);
			else SetForegroundWindow(w->bhwnd);
			InvalidateRect(hwnd, NULL, TRUE);
			w=NULL;
			free(w);
		}
		break;
	}

	return DefWindowProc(hwnd, msg, wParam, lParam);
}

void AddWindow(HWND hwnd)
{
	struct Window* w = (struct Window*)malloc(sizeof(struct Window));
	GetWindowText(hwnd, w->windowtext, 256);
	w->bhwnd = hwnd;
	w->index = COUNT++;
	w->hwnd = CreateWindowEx(0, "WPTaskWindow", w->windowtext, WS_VISIBLE | WS_BORDER | WS_CHILD | WS_CLIPSIBLINGS, 0, 0, 0, 0, hwndMain, NULL, hinst, 0);
	if (!w->hwnd) return;
	SetWindowLong(w->hwnd, GWL_USERDATA, (long)w);
	w=NULL;
	free(w);
	ResizeAllWindows();
}

void ResizeAllWindows()
{
	if (COUNT)
	{
		HWND hwnd=NULL;
		int width = (W-COUNT*HSPACING) / COUNT;
		int x=0;
		int y=0;
		int rowcount=0;

		if (width > TASK_WIDTH) width = TASK_WIDTH;
		x=0-width;

		while ((hwnd = FindWindowEx(hwndMain, hwnd, "WPTaskWindow", NULL)) != NULL)
		{
			struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);
		
			x += width + HSPACING;
			
			rowcount++;
			if (MAXROW != -1)
			{
				if (rowcount > MAXROW)
				{
					x = HSPACING;
					y += TASK_HEIGHT + VSPACING;
					rowcount=1;
				}
			}
			
			SetWindowPos(hwnd, NULL, x, y, width, TASK_HEIGHT, SWP_NOZORDER | SWP_NOACTIVATE);

			w=NULL;
			free(w);
		}
	}
	
	InvalidateRect(hwndMain, NULL, TRUE);
}

void WPChooseFont(HWND hwnd, char* CurrentFace, int CurrentHeight, COLORREF CurrentColor, char* DestFace, char* DestColor, char* DestHeight)
{
	CHOOSEFONT cf;
	LOGFONT lf;
	char temp[256] = "";
	
	strcpy(lf.lfFaceName, CurrentFace);
	lf.lfItalic = 0;
	lf.lfStrikeOut = 0;
	lf.lfUnderline = 0;
	lf.lfHeight = CurrentHeight;
	
	cf.lStructSize = sizeof(CHOOSEFONT);
	cf.hwndOwner = hwnd;
	cf.hDC = (HDC)NULL;
	cf.lpLogFont = &lf;
	cf.iPointSize = 0;
	cf.Flags = CF_SCREENFONTS | CF_EFFECTS | CF_INITTOLOGFONTSTRUCT;
	cf.rgbColors = CurrentColor;
	cf.lCustData = 0L;
	cf.lpfnHook = (LPCFHOOKPROC)NULL;
	cf.lpTemplateName = (LPSTR)NULL;
	cf.hInstance = (HINSTANCE) NULL;
	cf.lpszStyle = (LPSTR)NULL;
	cf.nFontType = SCREEN_FONTTYPE;
	cf.nSizeMin = 0;
	cf.nSizeMax = 0;
	
	if (ChooseFont(&cf))
	{
		char temp[256] = "";
		WritePrivateProfileString(szAppName, DestFace, lf.lfFaceName, cfgfile);
		sprintf(temp, "%lx", cf.rgbColors);
		WritePrivateProfileString(szAppName, DestColor, temp, cfgfile);
		sprintf(temp, "%d", cf.iPointSize/10);
		WritePrivateProfileString(szAppName, DestHeight, temp, cfgfile);
	}
}