#include <windows.h>
#include <stdio.h>
#include "resource.h"

__declspec( dllexport ) int InitModule(HINSTANCE hInstance, HWND parent, char* config);
__declspec( dllexport ) void QuitModule();
__declspec( dllexport ) DLGPROC DialogConfigProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
__declspec( dllexport ) char* QueryConfigId();
__declspec( dllexport ) char* QueryName();
__declspec( dllexport ) void ShowModuleWindow(DWORD show);

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK SmallWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

void AddWindow(HWND hwnd);
BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam);
void LoadSetup();
void MoveWindows(int OldDeskX, int NewDeskX, int OldDeskY, int NewDeskY);
void WPChooseColor(HWND hwnd, COLORREF* color, HANDLE* hbr);

HINSTANCE hinst=NULL;
HWND parent=NULL;
char cfgfile[MAX_PATH] = "";
char* szAppName = "WPVWM";

HWND hwndMain=NULL;

COLORREF nabg, abg, border, grid, window, windowborder;
HBRUSH nabgbrush=NULL, abgbrush=NULL, borderbrush=NULL;
HBRUSH windowbrush=NULL, windowborderbrush=NULL;
HPEN gridpen=NULL;

int X, Y, W, H, NumX, NumY;
int CURRENT_X=0, CURRENT_Y=0;
int ScreenX, ScreenY;
BOOL NOUPDATE=FALSE;
UINT WP_VWM_LEFT, WP_VWM_RIGHT, WP_VWM_UP, WP_VWM_DOWN;

const long magicDWord = 0x49474541;

struct Window
{
	HBITMAP bmp;
	HWND bhwnd;
	HWND hwnd;
	RECT br, r;

	char windowtext[256];

	int Desktop;
};

int InitModule(HINSTANCE hInstance, HWND Parent, char* config)
{
	// Save all incoming settings
	hinst = hInstance;
	parent = Parent;
	strcpy(cfgfile, config);

	if (parent)
	{
		WNDCLASS wc;
		
		memset(&wc,0,sizeof(wc));
		wc.hInstance = hInstance;
		wc.lpfnWndProc = WndProc;
		wc.lpszClassName = szAppName;
		wc.hCursor = LoadCursor(NULL, IDC_ARROW);
		RegisterClass(&wc);

		wc.lpfnWndProc = SmallWndProc;
		wc.lpszClassName = "WPVWMWindow";
		RegisterClass(&wc);

		LoadSetup();
	}

	return 1;
}

void QuitModule()
{
	// Unregister windows classes etc...
	KillTimer(hwndMain, 0);
	KillTimer(hwndMain, 1);
	UnregisterClass(szAppName, hinst);
	DestroyWindow(hwndMain);
	DeleteObject(nabgbrush);
	DeleteObject(abgbrush);
	DeleteObject(borderbrush);
	DeleteObject(gridpen);
	DeleteObject(windowbrush);
	DeleteObject(windowborderbrush);
}

void LoadSetup()
{
	char temp[256] = "";
	char* endstr;

	WP_VWM_LEFT = RegisterWindowMessage("!WPVWMLeft");
	WP_VWM_RIGHT = RegisterWindowMessage("!WPVWMRight");
	WP_VWM_UP = RegisterWindowMessage("!WPVWMUp");
	WP_VWM_DOWN = RegisterWindowMessage("!WPVWMDown");

	ScreenX = GetSystemMetrics(SM_CXSCREEN);
	ScreenY = GetSystemMetrics(SM_CYSCREEN);

	X = GetPrivateProfileInt(szAppName, "X", 0, cfgfile);
	Y = GetPrivateProfileInt(szAppName, "Y", 0, cfgfile);
	W = GetPrivateProfileInt(szAppName, "W", 0, cfgfile);
	H = GetPrivateProfileInt(szAppName, "H", 0, cfgfile);
	NumX = GetPrivateProfileInt(szAppName, "NumX", 3, cfgfile);
	NumY = GetPrivateProfileInt(szAppName, "NumY", 1, cfgfile);
	
	GetPrivateProfileString(szAppName, "NABGColor", "0", temp, 256, cfgfile);
	nabg = strtol( temp, &endstr, 16 );
	nabgbrush = CreateSolidBrush(nabg);
	
	GetPrivateProfileString(szAppName, "ABGColor", "0", temp, 256, cfgfile);
	abg = strtol( temp, &endstr, 16 );
	abgbrush = CreateSolidBrush(abg);
	
	GetPrivateProfileString(szAppName, "BorderColor", "0", temp, 256, cfgfile);
	border = strtol( temp, &endstr, 16 );
	borderbrush = CreateSolidBrush(border);

	GetPrivateProfileString(szAppName, "WindowColor", "0", temp, 256, cfgfile);
	window = strtol( temp, &endstr, 16 );
	windowbrush = CreateSolidBrush(window);

	GetPrivateProfileString(szAppName, "WindowBorderColor", "0", temp, 256, cfgfile);
	windowborder = strtol( temp, &endstr, 16 );
	windowborderbrush = CreateSolidBrush(windowborder);
	
	GetPrivateProfileString(szAppName, "GridColor", "0", temp, 256, cfgfile);
	grid = strtol( temp, &endstr, 16 );
	gridpen = CreatePen(PS_SOLID, 2, grid);

	hwndMain = CreateWindow(szAppName, szAppName, WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN, X, Y, W, H, parent, NULL, hinst, 0);

	EnumWindows(EnumWindowsProc, 0);

	SetTimer(hwndMain, 0, 100, NULL);
	SetTimer(hwndMain, 1, 500, NULL);
}

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	DWORD ws = GetWindowLong(hwnd, GWL_STYLE);
	DWORD wsex = GetWindowLong(hwnd, GWL_EXSTYLE);

	if (ws & WS_VISIBLE && hwnd != FindWindow("ProgMan", NULL))
	{
		HWND find = NULL;
		RECT r;

		while ((find = FindWindowEx(hwndMain, find, "WPVWMWindow", NULL)) != NULL)
		{
			struct Window* w = (struct Window*)GetWindowLong(find, GWL_USERDATA);
			if (hwnd == w->bhwnd) return TRUE;
		}

		GetWindowRect(hwnd, &r);
		SetWindowPos(hwnd, NULL, r.left - (ScreenX * (r.left / ScreenX)), r.top - (ScreenY * (r.top / ScreenY)), 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);

		AddWindow(hwnd);
	}

	return TRUE;
}

DLGPROC DialogConfigProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_DRAWITEM:
		{
			LPDRAWITEMSTRUCT dis = (LPDRAWITEMSTRUCT)lParam;
			HBITMAP bmp = CreateCompatibleBitmap(dis->hDC, 10, 10);
			HBITMAP oldBmp = (HBITMAP)SelectObject(dis->hDC, bmp);
			RECT cr, r;

			GetClientRect(dis->hwndItem, &cr);
			if (dis->itemAction != ODA_SELECT) DrawEdge(dis->hDC, &cr, EDGE_RAISED, BF_RECT);
			else DrawEdge(dis->hDC, &cr, EDGE_SUNKEN, BF_RECT);
			SetRect(&r, cr.left+5, cr.top+5, cr.right-5, cr.bottom-5);
			
			if (dis->CtlID == IDC_COLOR_INACTIVE_BACKGROUND) FillRect(dis->hDC, &r, nabgbrush);
			else if (dis->CtlID == IDC_COLOR_ACTIVE_BACKGROUND) FillRect(dis->hDC, &r, abgbrush);
			else if (dis->CtlID == IDC_COLOR_BORDER) FillRect(dis->hDC, &r, borderbrush);
			else if (dis->CtlID == IDC_COLOR_WINDOW) FillRect(dis->hDC, &r, windowbrush);
			else if (dis->CtlID == IDC_COLOR_WINDOW_BORDER) FillRect(dis->hDC, &r, windowborderbrush);
			else if (dis->CtlID == IDC_COLOR_GRID) 
			{
				SelectObject(dis->hDC, gridpen);
				MoveToEx(dis->hDC, cr.right/2, cr.top + 5, NULL);
				LineTo(dis->hDC, cr.right/2, cr.bottom - 5);
				MoveToEx(dis->hDC, 5, cr.bottom/2, NULL);
				LineTo(dis->hDC, cr.right-5, cr.bottom/2);
			}

			SelectObject(dis->hDC, oldBmp);
			DeleteObject(bmp);
		}
		return (DLGPROC)TRUE;

		case WM_COMMAND:
		{
			switch (HIWORD(wParam))
			{
				case BN_CLICKED:
				{
					switch (LOWORD(wParam))
					{
						case IDC_COLOR_INACTIVE_BACKGROUND: WPChooseColor(hwnd, &nabg, &nabgbrush); break;
						case IDC_COLOR_ACTIVE_BACKGROUND: WPChooseColor(hwnd, &abg, &abgbrush); break;
						case IDC_COLOR_BORDER: WPChooseColor(hwnd, &border, &borderbrush); break;
						case IDC_COLOR_WINDOW: WPChooseColor(hwnd, &window, &windowbrush); break;
						case IDC_COLOR_WINDOW_BORDER: WPChooseColor(hwnd, &windowborder, &windowborderbrush); break;
						case IDC_COLOR_GRID: WPChooseColor(hwnd, &grid, &gridpen); break;
					}
				}
				break;
			}
		}
		break;

		case WM_INITDIALOG:
		{
			SetDlgItemInt(hwnd, IDC_NUM_X, NumX, FALSE);
			SetDlgItemInt(hwnd, IDC_NUM_Y, NumY, FALSE);
			SetDlgItemInt(hwnd, IDC_X, X, TRUE);
			SetDlgItemInt(hwnd, IDC_Y, Y, TRUE);
			SetDlgItemInt(hwnd, IDC_WIDTH, W, FALSE);
			SetDlgItemInt(hwnd, IDC_HEIGHT, H, FALSE);
		}
		break;

		case WM_DESTROY:
		{
			char temp[256] = "";
			
			sprintf(temp, "%lx", nabg);
			_strupr(temp);
			WritePrivateProfileString(szAppName, "NABGColor", temp, cfgfile);

			sprintf(temp, "%lx", abg);
			_strupr(temp);
			WritePrivateProfileString(szAppName, "ABGColor", temp, cfgfile);

			sprintf(temp, "%lx", border);
			_strupr(temp);
			WritePrivateProfileString(szAppName, "BorderColor", temp, cfgfile);

			sprintf(temp, "%lx", window);
			_strupr(temp);
			WritePrivateProfileString(szAppName, "WindowColor", temp, cfgfile);

			sprintf(temp, "%lx", windowborder);
			_strupr(temp);
			WritePrivateProfileString(szAppName, "WindowBorderColor", temp, cfgfile);

			sprintf(temp, "%lx", grid);
			_strupr(temp);
			WritePrivateProfileString(szAppName, "GridColor", temp, cfgfile);

			GetDlgItemText(hwnd, IDC_NUM_X, temp, 256);
			WritePrivateProfileString(szAppName, "NumX", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_NUM_Y, temp, 256);
			WritePrivateProfileString(szAppName, "NumY", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_X, temp, 256);
			WritePrivateProfileString(szAppName, "X", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_Y, temp, 256);
			WritePrivateProfileString(szAppName, "Y", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_WIDTH, temp, 256);
			WritePrivateProfileString(szAppName, "W", temp, cfgfile);
			GetDlgItemText(hwnd, IDC_HEIGHT, temp, 256);
			WritePrivateProfileString(szAppName, "H", temp, cfgfile);
		}
		break;
	}

	return FALSE;
}

char* QueryConfigId() { return MAKEINTRESOURCE(IDD_CONFIG); }
char* QueryName() { return szAppName; }
void ShowModuleWindow(DWORD show) 
{ 
	ShowWindow(hwndMain, show);
	if (show == SW_SHOW)
	{
		HWND hwnd=NULL;
		while ((hwnd = FindWindowEx(hwndMain, hwnd, "WPVWMWindow", NULL)) != NULL)
		{
			InvalidateRect(hwnd, NULL, TRUE);
		}
	}
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd, &ps);
			HDC buf = CreateCompatibleDC(NULL);
			HBITMAP bufbmp = CreateCompatibleBitmap(hdc, W, H), oldBmp;
			RECT r, vw;
			int x=0, y=0;

			oldBmp = (HBITMAP)SelectObject(buf, bufbmp);

			GetClientRect(hwnd, &r);
			FrameRect(buf, &r, borderbrush);

			SelectObject(buf, gridpen);
			for (x=1; x < NumX; x++)
			{
				MoveToEx(buf, r.left + (r.right / NumX * x), 1, NULL);
				LineTo(buf, r.left + (r.right / NumX * x), r.bottom - 1);
			}

			for (x=1; x < NumY; x++)
			{
				MoveToEx(buf, 1, r.top + (r.bottom / NumY * x), NULL);
				LineTo(buf, r.right-1, r.top + (r.bottom / NumY * x));
			}

			for (y=0; y < NumY; y++)
			{
				for (x=0; x < NumX; x++)
				{
					SetRect(&vw, 
						r.left + (r.right / NumX * x) + 1, 
						r.top + (r.bottom / NumY * y) + 1, 
						r.left + r.right / NumX * (x+1) - 1, 
						r.top + r.bottom / NumY * (y+1) - 1);

					if (x == CURRENT_X && y == CURRENT_Y) 
					{
						//HDC desktop = GetDC(GetDesktopWindow());
						//StretchBlt(hdc, vw.left, vw.top, vw.right, vw.bottom, desktop, 0, 0, ScreenX, ScreenY, SRCCOPY);
						//ReleaseDC(GetDesktopWindow(), desktop);
						FillRect(buf, &vw, abgbrush);
					}
					else FillRect(buf, &vw, nabgbrush);
				}
			}

			BitBlt(hdc, 0, 0, W, H, buf, 0, 0, SRCCOPY);

			SelectObject(buf, oldBmp);
			DeleteObject(bufbmp);
			DeleteDC(buf);
			EndPaint(hwnd, &ps);
		}
		break;

		case WM_RBUTTONUP:
		{
			int xpos = LOWORD(lParam);
			int ypos = HIWORD(lParam);
			int deskx = xpos / (W/NumX);
			int desky = ypos / (H/NumY);

			if (deskx != CURRENT_X || desky != CURRENT_Y)
			{
				MoveWindows(CURRENT_X, deskx, CURRENT_Y, desky);
				CURRENT_X = deskx;
				CURRENT_Y = desky;
				InvalidateRect(hwnd, NULL, TRUE);
			}
		}
		break;

		case WM_TIMER:
		{
			switch ((int)wParam)
			{
				case 0:
				{
					HWND hwnd = NULL;

					while ((hwnd = FindWindowEx(hwndMain, hwnd, "WPVWMWindow", NULL)) != NULL)
					{
						int width, height, x, y;
						struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);

						if (w)
						{
							if (IsWindow(w->bhwnd) && IsWindowVisible(w->bhwnd))
							{
								RECT wm;
								GetWindowRect(w->bhwnd, &w->br);
								GetWindowRect(hwndMain, &wm);
								GetWindowRect(hwnd, &w->r);
								
								x = (w->br.left + (ScreenX * CURRENT_X)) * W / NumX / ScreenX;
								y = (w->br.top + (ScreenY * CURRENT_Y)) * H / NumY / ScreenY;
								//y = w->br.top * H / ScreenY;
								width = (w->br.right - w->br.left) / NumX *  W / ScreenX;
								height = (w->br.bottom - w->br.top ) / NumY * H / ScreenY;
								//height = (w->br.bottom - w->br.top) * H / ScreenY;
								
								if (w->r.right - w->r.left != width ||
									w->r.bottom - w->r.top != height ||
									w->r.left - wm.left != x ||
									w->r.top - wm.top != y)
								{
									NOUPDATE=TRUE;
									SetWindowPos(hwnd, NULL, x, y, width, height, SWP_NOZORDER | SWP_NOACTIVATE);
									InvalidateRect(hwndMain, NULL, TRUE);
									InvalidateRect(hwnd, NULL, TRUE);
								}
								else if (w->bhwnd == GetForegroundWindow())
								{
									SetWindowPos(hwnd, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
								}
							}
							else
							{
								DestroyWindow(w->hwnd);
								DeleteObject(w->bmp);
							}
						}

						w=NULL;
						free(w);
					}
				}
				
				case 1:
				{
					EnumWindows(EnumWindowsProc, 0);
				}
				break;
			}
		}
		break;

		default:
		{
			if (msg == WP_VWM_RIGHT)
			{
				if (CURRENT_X+1 == NumX)
				{
					MoveWindows(CURRENT_X, 0, CURRENT_Y, CURRENT_Y);
					CURRENT_X=0;
				}
				else
				{
					MoveWindows(CURRENT_X, CURRENT_X+1, CURRENT_Y, CURRENT_Y);
					CURRENT_X++;
				}
			}
			else if (msg == WP_VWM_LEFT)
			{
				if (CURRENT_X == 0)
				{
					MoveWindows(CURRENT_X, NumX-1, CURRENT_Y, CURRENT_Y);
					CURRENT_X = NumX-1;
				}
				else
				{
					MoveWindows(CURRENT_X, CURRENT_X-1, CURRENT_Y, CURRENT_Y);
					CURRENT_X--;
				}
			}
			else if (msg == WP_VWM_UP)
			{
				if (CURRENT_Y == 0)
				{
					MoveWindows(CURRENT_X, CURRENT_X, CURRENT_Y, NumY-1);
					CURRENT_Y = NumY-1;
				}
				else
				{
					MoveWindows(CURRENT_X, CURRENT_X, CURRENT_Y, CURRENT_Y-1);
					CURRENT_Y--;
				}
			}
			else if (msg == WP_VWM_DOWN)
			{
				if (CURRENT_Y+1 == NumY)
				{
					MoveWindows(CURRENT_X, CURRENT_X, CURRENT_Y, 0);
					CURRENT_Y = 0;
				}
				else
				{
					MoveWindows(CURRENT_X, CURRENT_X, CURRENT_Y, CURRENT_Y+1);
					CURRENT_Y++;
				}
			}
		}
		break;
	}

	return DefWindowProc(hwnd, msg, wParam, lParam);
}

LRESULT CALLBACK SmallWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_NCHITTEST: return HTCAPTION;

		case WM_NCPAINT:
		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd, &ps);
			RECT r;
			GetClientRect(hwnd, &r);
			FillRect(hdc, &r, windowbrush);
			FrameRect(hdc, &r, windowborderbrush);

			//HDC buf = CreateCompatibleDC(NULL);
			//HBITMAP oldBmp=NULL;
			//struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);
			
			//if (!w->bmp)
			//{
			//	HDC hwndDC = GetDC(w->bhwnd);
			//	w->bmp = CreateCompatibleBitmap(hwndDC, w->br.right - w->br.left, w->br.bottom - w->br.top);
			//	oldBmp=(HBITMAP)SelectObject(buf, w->bmp);
			//	BitBlt(buf, 0, 0, w->br.right - w->br.left, w->br.bottom - w->br.top, hwndDC, 0, 0, SRCCOPY);
			//	ReleaseDC(w->bhwnd, hwndDC);
			//}
			//else
			//{
			//	oldBmp=(HBITMAP)SelectObject(buf, w->bmp);
			//}
			
			//StretchBlt(hdc, 0, 0, w->r.right, w->r.bottom, buf, 0, 0, w->br.right - w->br.left, w->br.bottom - w->br.top, SRCCOPY);

			//SelectObject(buf, oldBmp);
			//DeleteDC(buf);
			EndPaint(hwnd, &ps);
			//w=NULL;
			//free(w);
		}
		break;

		case WM_NCRBUTTONUP:
		case WM_RBUTTONUP:
		{
			RECT sr, mr;
			int x, y;
			GetWindowRect(hwndMain, &mr);
			GetWindowRect(hwnd, &sr);

			x = LOWORD(lParam) - mr.left;
			y = HIWORD(lParam) - mr.top;

			PostMessage(hwndMain, WM_RBUTTONUP, wParam, MAKELPARAM(x, y));
		}
		break;

		case WM_NCLBUTTONUP:
		case WM_LBUTTONUP:
		{
			struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);

			SetForegroundWindow(w->bhwnd);

			w=NULL;
			free(w);
		}
		return 0;

		case WM_WINDOWPOSCHANGING:
		{
			struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);

			if (w && !NOUPDATE)
			{
				LPWINDOWPOS wp = (LPWINDOWPOS)lParam;

				if (!(wp->flags & SWP_NOMOVE))
				{
					int x = (ScreenX * wp->x / W * NumX) + (w->Desktop - CURRENT_X) * ScreenX;
					int y = (ScreenY * wp->y / H * NumY) + (w->Desktop - CURRENT_Y) * ScreenY;
					//int y = ScreenY * wp->y / H;

					wp->hwndInsertAfter = HWND_TOP;

					SetWindowPos(w->bhwnd, NULL, x, y, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
					GetWindowRect(w->bhwnd, &w->br);
					InvalidateRect(hwnd, NULL, TRUE);
				}
			}

			NOUPDATE=FALSE;
			w=NULL;
			free(w);
		}
		break;
	}

	return DefWindowProc(hwnd, msg, wParam, lParam);
}

void WPChooseColor(HWND hwnd, COLORREF* color, HANDLE* hbr)
{
	CHOOSECOLOR cc;
	static COLORREF acrCustClr[16];
	
	ZeroMemory(&cc, sizeof(CHOOSECOLOR));
	cc.lStructSize = sizeof(CHOOSECOLOR);
	cc.hwndOwner = hwnd;
	cc.lpCustColors = (LPDWORD) acrCustClr;
	cc.rgbResult = *color;
	cc.Flags = CC_FULLOPEN | CC_RGBINIT; 
	
	if (ChooseColor(&cc)==TRUE) 
	{
		DeleteObject(&hbr);
		*color = cc.rgbResult;
		if (*hbr != gridpen) *hbr = (HBRUSH)CreateSolidBrush(*color);
		else *hbr = (HBRUSH)CreatePen(PS_SOLID, 2, *color);
		InvalidateRect(hwnd, NULL, TRUE);
	}
}

void MoveWindows(int OldDeskX, int NewDeskX, int OldDeskY, int NewDeskY)
{
	HWND hwnd = NULL;

	while ((hwnd = FindWindowEx(hwndMain, hwnd, "WPVWMWindow", NULL)) != NULL)
	{
		struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);

		if (!GetWindowLong(w->bhwnd, GWL_USERDATA) & magicDWord)
		{
			GetWindowRect(w->bhwnd, &w->br);
			SetWindowPos(w->bhwnd, NULL, 
				w->br.left + (OldDeskX - NewDeskX) * ScreenX, 
				w->br.top + (OldDeskY - NewDeskY) * ScreenY, 
				0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
			GetWindowRect(w->bhwnd, &w->br);
		}
	}
}

void AddWindow(HWND hwnd)
{
	struct Window* w = (struct Window*)GetWindowLong(hwnd, GWL_USERDATA);
	
	if (!w)
	{
		w = (struct Window*)malloc(sizeof(struct Window));
		
		GetWindowRect(hwnd, &w->br);
		GetWindowText(hwnd, w->windowtext, 256);
		w->bhwnd = hwnd;
		
		w->hwnd = CreateWindowEx(0, "WPVWMWindow", w->windowtext, WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS, 
			w->br.left * W / NumX / ScreenX, 
			w->br.top * H / NumY / ScreenY, 
			(w->br.right - w->br.left) / NumX *  W / ScreenX, 
			(w->br.bottom - w->br.top) / NumY * H / ScreenY, 
			hwndMain, NULL, hinst, 0);

		if (!w->hwnd) return;
		
		GetClientRect(w->hwnd, &w->r);
		w->Desktop = 0;
		
		SetWindowLong(w->hwnd, GWL_USERDATA, (long)w);
	}
	
	w=NULL;
	free(w);
}